/* This file is part of Mailfromd.
   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011 Sergey
   Poznyakoff

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <mailutils/mailutils.h>
#include <mailutils/libargp.h>
#include <mailutils/gocs.h>
#include "libmf.h"

extern struct mu_cfg_tree *mu_argp_tree;

char *mu_app_rcfile;

static char *
get_canonical_name ()
{
  char *name;
  size_t len;
  char *p = strchr (argp_program_version, ' ');
  if (!p)
    return strdup (mu_program_name);
  len = p - argp_program_version;
  name = malloc (len + 1);
  if (!name)
    abort ();
  memcpy (name, argp_program_version, len);
  name[len] = 0;
  return name;
}

static void
fixup_capa (char **excapa)
{
  char **p, **q;

  for (p = q = excapa; *q; q++)
    {
      if (**q != '.')
	*p++ = *q;
    }
  *p = NULL;
}
  
int
mfd_app_init (struct argp *myargp, const char **capa,
	      struct mu_cfg_param *cfg_param,
	      int argc, char **argv, int flags, int *pindex,
	      void *opt_data, void *cfg_data)
{
  int rc, i;
  struct argp *argp;
  struct argp argpnull = { 0 };
  char **excapa;
  struct mu_cfg_tree *parse_tree = NULL;
  struct mu_cfg_parse_hints hints = { MU_PARSE_CONFIG_PLAIN };
  
  mu_set_program_name (argv[0]);
  mu_log_tag = (char*)mu_program_name;
  mu_stdstream_setup ();
  mu_libargp_init ();
  if (capa)
    for (i = 0; capa[i]; i++)
      mu_gocs_register_std (capa[i]); 
  if (!myargp)
    myargp = &argpnull;
  argp = mu_argp_build (myargp, &excapa);

  fixup_capa (excapa);
  
  mu_cfg_tree_create (&mu_argp_tree);
  rc = argp_parse (argp, argc, argv, flags, pindex, opt_data);
  mu_argp_done (argp);
  if (rc)
    return rc;

  /* Reset program name, it may have been changed using the `--program-name'
     option. */
  mu_set_program_name (program_invocation_name);
  
  mu_libcfg_init (excapa);
  free (excapa);

  if (mu_help_config_mode)
    {
      char *comment;
      char *canonical_name = get_canonical_name ();
      mu_stream_t stream;
      mu_stdio_stream_create (&stream, MU_STDOUT_FD, 0);
      asprintf (&comment,
		_("This is a template for %s: the configuration file for %s."),
		mu_app_rcfile, canonical_name);
      mu_cfg_format_docstring (stream, comment, 0);
      free (comment);
      asprintf (&comment, "For more information, use `info %s'.",
		canonical_name);
      mu_cfg_format_docstring (stream, comment, 0);
      free (comment);
      
      mu_format_config_tree (stream, cfg_param);
      mu_stream_destroy (&stream);
      exit (0);
    }

  /* Set up hints */
  if (mu_cfg_parser_verbose)
    hints.flags |= MU_PARSE_CONFIG_VERBOSE;
  if (mu_cfg_parser_verbose > 1)
    hints.flags |= MU_PARSE_CONFIG_DUMP;

  if (mu_load_site_rcfile)
    {
      hints.flags |= MU_CFG_PARSE_SITE_RCFILE;
      hints.site_rcfile = mu_app_rcfile;
    }
  if (mu_load_user_rcfile && mu_program_name)
    {
      hints.flags |= MU_CFG_PARSE_PROGRAM;
      hints.program = (char*) mu_program_name;
    }
  if (mu_load_rcfile)
    {
      hints.flags |= MU_CFG_PARSE_CUSTOM_RCFILE;
      hints.custom_rcfile = mu_load_rcfile;
    }
  
  rc = mu_cfg_parse_config (&parse_tree, &hints);
  if (rc == 0)
    {
      if (mu_cfg_parser_verbose)
	hints.flags |= MU_PARSE_CONFIG_VERBOSE;
      if (mu_cfg_parser_verbose > 1)
	hints.flags |= MU_PARSE_CONFIG_DUMP;
      mu_cfg_tree_postprocess (mu_argp_tree, &hints);
      mu_cfg_tree_union (&parse_tree, &mu_argp_tree);
      rc = mu_cfg_tree_reduce (parse_tree, &hints, cfg_param, cfg_data);
    }
  
  if (mu_rcfile_lint)
    {
      if (rc || mu_cfg_error_count)
	exit (1);
      if (mu_app_cfg_verifier)
	rc = mu_app_cfg_verifier ();
      exit (rc ? 1 : 0);
    }
  
  mu_gocs_flush ();
  mu_cfg_destroy_tree (&mu_argp_tree);
  mu_cfg_destroy_tree (&parse_tree);

  return !!(rc || mu_cfg_error_count);
}

